@@ -14,7 +14,6 @@ import warnings
 from collections import defaultdict
 
 import six
-import pkg_resources
 import requests
 
 try:
@@ -33,10 +32,10 @@ from copr.v3 import (
     CoprConfigException, CoprNoResultException, CoprAuthException,
 )
 from copr.v3.pagination import next_page
-from copr_cli.helpers import cli_use_output_format
+from copr_cli.helpers import cli_use_output_format, print_project_info
 from copr_cli.monitor import cli_monitor_parser
 from copr_cli.printers import cli_get_output_printer as get_printer
-from copr_cli.util import get_progress_callback, serializable
+from copr_cli.util import get_progress_callback, serializable, package_version
 from .build_config import MockProfile
 
 
@@ -147,43 +146,39 @@ class Commands(object):
     @property
     def username(self):
         """
-        Get the username from config, or obtain it via auth_check (transitively
-        via GSSAPI).
+        Get the username from user's config, or obtain it via auth_check
+        (transitively via GSSAPI).
         """
         if self.config.get("username"):
             return self.config["username"]
 
         if self.config.get("gssapi"):
-            try:
-                return self.client.base_proxy.auth_username()
-            except CoprAuthException:
-                log.error("Failed to determine Copr username from authentication.")
-                raise
+            return self.client.base_proxy.auth_username()
         raise CoprConfigException(
             "This operation tries to detect your username, but it is not "
             "possible to find it in configuration, and GSSAPI is disabled "
         )
 
-    @property
-    def ownername(self):
-        """
-        Determine the project ownername (== username) when not specified on
-        commandline.
-        """
-        try:
-            return self.username
-        except:
-            log.error("This operation needs a project ownername specified, "
-                      "fallback to your username failed.")
-            raise
-
     def parse_name(self, name):
+        """
+        Several sub-commands accept the project NAME argument in the '<project>'
+        (that defaults to '<authenticated_user>/<project>'), or in the full
+        '<owner>/<project>' format.  Translate this NAME string to the
+        (owner, project) pair.
+        """
         m = re.match(r"([^/]+)/(.*)", name)
         if m:
             owner = m.group(1)
             name = m.group(2)
         else:
-            owner = self.ownername
+            try:
+                owner = self.username
+            except:
+                log.error("Wrong project argument format '%s'.  Please use "
+                          "the full '<owner>/<project>' argument format, or "
+                          "authenticate to use the short '<project>' format "
+                          "(which means '<your_username>/<project>').", name)
+                raise
         return owner, name
 
     def parse_dirname(self, name):
@@ -211,7 +206,18 @@ class Commands(object):
         """
         m = re.match(r"(([^/]+)/)?([^/]+)/(.*)", path)
         if m:
-            owner = m.group(2) or self.username
+            owner = m.group(2)
+            if not owner:
+                try:
+                    owner = self.username
+                except:
+                    log.error("Wrong chroot path format '%s'. Use the full "
+                              "'<owner>/<project>/<chroot>' format or "
+                              "authenticate to use the short "
+                              "'<project>/<chroot>' format (which means "
+                              "'<your_username>/<project>/<chroot>').", path)
+                    raise
+
             return owner, m.group(3), m.group(4)
         raise CoprException("Unexpected chroot path format")
 
@@ -279,7 +285,11 @@ class Commands(object):
         """
         Simply print out the current user as defined in copr config.
         """
-        print(self.username)
+        try:
+            print(self.username)
+        except:
+            log.error("Can't detect who are you.")
+            raise
 
     def action_new_webhook_secret(self, args):
         """
@@ -308,17 +318,23 @@ class Commands(object):
         :param args: argparse arguments provided by the user
         """
 
-        # Before we start uploading potentially large source RPM file, make sure
-        # that the user actually has a valid credentials.
-        self.client.base_proxy.auth_check()
-
         username, projectname, project_dirname = self.parse_dirname(args.copr_repo)
+        buildopts = buildopts_from_args(args)
+
+        # Before we start uploading potentially large source RPM file, make sure
+        # that the user has valid credentials and can build in the project.
+        self.client.build_proxy.check_before_build(
+            ownername=username,
+            projectname=projectname,
+            project_dirname=project_dirname,
+            buildopts=buildopts,
+        )
 
         builds = []
         for pkg in args.pkgs:
             if os.path.exists(pkg):
                 progress_callback = get_progress_callback(os.path.getsize(pkg))
-                buildopts = buildopts_from_args(args, progress_callback)
+                buildopts["progress_callback"] = progress_callback
                 data = {"path": pkg}
                 print('Uploading package {0}'.format(pkg))
                 try:
@@ -332,7 +348,6 @@ class Commands(object):
             elif not urlparse(pkg).scheme:
                 raise CoprException("File {0} not found".format(pkg))
             else:
-                buildopts = buildopts_from_args(args)
                 data = {"url": pkg}
                 builds.append(self.client.build_proxy.create_from_url(
                     ownername=username, projectname=projectname,
@@ -461,6 +476,7 @@ class Commands(object):
             auto_prune=ON_OFF_MAP[args.auto_prune],
             bootstrap=BOOTSTRAP_MAP[args.bootstrap],
             isolation=args.isolation,
+            follow_fedora_branching=ON_OFF_MAP[args.follow_fedora_branching],
             delete_after_days=args.delete_after_days,
             multilib=ON_OFF_MAP[args.multilib],
             module_hotfixes=ON_OFF_MAP[args.module_hotfixes],
@@ -468,6 +484,8 @@ class Commands(object):
             appstream=ON_OFF_MAP[args.appstream],
             runtime_dependencies=args.runtime_dependencies,
             packit_forge_projects_allowed=args.packit_forge_projects_allowed,
+            repo_priority=args.repo_priority,
+            storage=args.storage,
         )
 
         owner_part = username.replace('@', "g/")
@@ -492,6 +510,7 @@ class Commands(object):
             auto_prune=ON_OFF_MAP[args.auto_prune],
             bootstrap=BOOTSTRAP_MAP[args.bootstrap],
             isolation=args.isolation,
+            follow_fedora_branching=ON_OFF_MAP[args.follow_fedora_branching],
             chroots=args.chroots,
             delete_after_days=args.delete_after_days,
             multilib=ON_OFF_MAP[args.multilib],
@@ -500,6 +519,7 @@ class Commands(object):
             appstream=ON_OFF_MAP[args.appstream],
             runtime_dependencies=args.runtime_dependencies,
             packit_forge_projects_allowed=args.packit_forge_projects_allowed,
+            repo_priority=args.repo_priority,
         )
 
     @requires_api_auth
@@ -588,7 +608,6 @@ class Commands(object):
         build_config.rootdir = "{0}-{1}_{2}".format(ownername.replace("@", "group_"), projectname, args.chroot)
         print(MockProfile(build_config))
 
-
     def action_list(self, args):
         """ Method called when the 'list' action has been selected by the
         user.
@@ -596,22 +615,21 @@ class Commands(object):
         :param args: argparse arguments provided by the user
 
         """
-        username = args.username or self.ownername
+        username = args.username
+        if not username:
+            try:
+                username = self.username
+            except:
+                log.error("The 'username|@groupname' not specified.  Either "
+                          "specify it, or authenticate to list your projects.")
+                raise
         projects = self.client.project_proxy.get_list(username)
         if not projects:
             sys.stderr.write("No copr retrieved for user: '{0}'\n".format(username))
             return
 
         for project in projects:
-            print("Name: {0}".format(project.name))
-            print("  Description: {0}".format(project.description))
-            if project.chroot_repos:
-                print("  Repo(s):")
-                for name, url in project.chroot_repos.items():
-                    print("    {0}: {1}".format(name, url))
-            if project.additional_repos:
-                print("  Additional repo: {0}".format(" ".join(project.additional_repos)))
-            print("")
+            print_project_info(project)
 
     def action_status(self, args):
         build = self.client.build_proxy.get(args.build_id)
@@ -640,6 +658,19 @@ class Commands(object):
             if args.spec:
                 cmd.extend(["-A", "*.spec"])
 
+            if args.logs:
+                cmd.extend(["-A", "*.log.gz"])
+
+            if args.review:
+                cmd.extend([
+                    "-A", "files.dir",
+                    "-A", "licensecheck.txt",
+                    "-A", "review.txt",
+                    "-A", "review.json",
+                    "-A", "rpmlint.txt",
+                ])
+                cmd.append(chroot.result_url + "fedora-review")
+
             cmd.append(chroot.result_url)
             subprocess.call(cmd)
 
@@ -659,6 +690,15 @@ class Commands(object):
         result = self.client.build_proxy.delete_list(args.build_id)
         print("Build(s) {0} were deleted.".format(", ".join(map(str, result["builds"]))))
 
+    def action_get(self, args):
+        """ Method called when the 'get' action has been selected by the
+        user.
+
+        :param args: argparse arguments provided by the user
+        """
+        owner, project = self.parse_name(args.project)
+        print_project_info(self.client.project_proxy.get(owner, project))
+
     #########################################################
     ###                   Chroot actions                  ###
     #########################################################
@@ -706,7 +746,8 @@ class Commands(object):
         )
         fields = ["additional_packages", "additional_repos", "additional_modules",
                   "comps_name", "delete_after_days", "isolation", "mock_chroot",
-                  "ownername", "projectname", "with_opts", "without_opts"]
+                  "ownername", "projectname", "with_opts", "without_opts",
+                  "follow_fedora_branching"]
         printer = get_printer(args.output_format, fields)
         printer.add_data(project_chroot)
         printer.finish()
@@ -739,6 +780,7 @@ class Commands(object):
             "spec_template": args.spec_template,
             "python_versions": args.pythonversions,
             "max_builds": args.max_builds,
+            "timeout": args.timeout,
             "webhook_rebuild": ON_OFF_MAP[args.webhook_rebuild],
         }
         if args.create:
@@ -759,6 +801,7 @@ class Commands(object):
             "scm_type": args.scm_type,
             "source_build_method": args.srpm_build_method,
             "max_builds": args.max_builds,
+            "timeout": args.timeout,
             "webhook_rebuild": ON_OFF_MAP[args.webhook_rebuild],
         }
         if args.create:
@@ -776,6 +819,7 @@ class Commands(object):
             "namespace": args.namespace,
             "committish": args.committish,
             "max_builds": args.max_builds,
+            "timeout": args.timeout,
             "webhook_rebuild": ON_OFF_MAP[args.webhook_rebuild],
         }
         ownername, projectname = self.parse_name(args.copr)
@@ -794,6 +838,7 @@ class Commands(object):
             "package_name": args.name,
             "gem_name": args.gem_name,
             "max_builds": args.max_builds,
+            "timeout": args.timeout,
             "webhook_rebuild": ON_OFF_MAP[args.webhook_rebuild],
         }
         if args.create:
@@ -812,6 +857,7 @@ class Commands(object):
             "script_builddeps": args.script_builddeps,
             "script_resultdir": args.script_resultdir,
             "max_builds": args.max_builds,
+            "timeout": args.timeout,
             "webhook_rebuild": ON_OFF_MAP[args.webhook_rebuild],
         }
         if args.create:
@@ -1019,7 +1065,7 @@ def setup_parser():
                         help="Path to an alternative configuration file")
 
     parser.add_argument("--version", action="version",
-                        version="%(prog)s version " + pkg_resources.require('copr-cli')[0].version)
+                        version="%(prog)s version " + package_version("copr-cli"))
 
     subparsers = parser.add_subparsers(title="actions")
 
@@ -1069,7 +1115,8 @@ def setup_parser():
     parser_create = subparsers.add_parser("create", help="Create a new copr")
     parser_create.add_argument("name",
                                help="The name of the copr to create")
-    parser_create.add_argument("--chroot", dest="chroots", action="append",
+    parser_create.add_argument("--chroot", dest="chroots", metavar="CHROOT",
+                               action="append",
                                help="Chroot to use for this copr")
 
 
@@ -1134,6 +1181,25 @@ def setup_parser():
         help=("Generate AppStream metadata for this project. Generating "
               "metadata slows down the builds in large Copr projects."))
 
+    parser_create.add_argument(
+        "--follow-fedora-branching", choices=["on", "off"], default="on",
+        help=("When Fedora is branched from rawhide, the "
+              "respective chroots for the new branch are automatically "
+              "created for you (as soon as they are available) as rawhide "
+              "chroot forks."))
+
+    parser_create.add_argument(
+        "--repo-priority", default=None,
+        help=("Use the priority=<INT> config option for repositories in this "
+              "project, see man dnf.conf(5) for more info."))
+
+    parser_create.add_argument(
+        "--storage",
+        choices=["backend", "pulp"],
+        help=("What storage should be used for this project. "
+              "This option can only be specified by a COPR admin.")
+    )
+
     create_and_modify_common_opts(parser_create)
 
     parser_create.set_defaults(func="action_create")
@@ -1142,7 +1208,8 @@ def setup_parser():
     parser_modify = subparsers.add_parser("modify", help="Modify existing copr")
 
     parser_modify.add_argument("name", help="The name of the copr to modify")
-    parser_modify.add_argument("--chroot", dest="chroots", action="append",
+    parser_modify.add_argument("--chroot", dest="chroots", metavar="CHROOT",
+                               action="append",
                                help="Chroot to use for this copr")
     parser_modify.add_argument("--description",
                                help="Description of the copr")
@@ -1182,17 +1249,27 @@ def setup_parser():
               "x86_64 arch), default is \"don't change\""))
 
     parser_modify.add_argument(
-        "--fedora-review", choices=["on", "off"], default="off",
+        "--fedora-review", choices=["on", "off"],
         help=("When submitting new package to Fedora, it needs to comply with "
               "Fedora Packaging Guidelines. Use fedora-review tool to help you "
               "discover packaging errors. Failing fedora-review will not fail "
               "the build itself."))
 
     parser_modify.add_argument(
-        "--appstream", choices=["on", "off"], default="off",
+        "--appstream", choices=["on", "off"],
         help=("Generate AppStream metadata for this project. Generating "
               "metadata slows down the builds in large Copr projects."))
 
+    parser_modify.add_argument(
+        "--follow-fedora-branching", choices=["on", "off"],
+        help=("When Fedora is branched from rawhide, the "
+              "respective chroots for the new branch are automatically "
+              "created for you (as soon as they are available) as rawhide "
+              "chroot forks."))
+
+    parser_modify.add_argument("--repo-priority", default=None,
+                               help="Set the priority value of this repository")
+
     create_and_modify_common_opts(parser_modify)
 
     parser_modify.set_defaults(func="action_modify_project")
@@ -1226,6 +1303,11 @@ def setup_parser():
     cli_use_output_format(parser_builds, default=None)
     parser_builds.set_defaults(func="action_list_builds")
 
+    parser_get = subparsers.add_parser("get", help="Get information about project")
+    parser_get.add_argument("project", help="Which project you want to get. Can be in format "
+                            "`username/project` or in case of group `@groupname/project`")
+    parser_get.set_defaults(func="action_get")
+
     #########################################################
     ###             Source-type related options           ###
     #########################################################
@@ -1241,7 +1323,7 @@ def setup_parser():
         dest="spec_generator",
         help="Tool for generating specfile from a PyPI package",
         choices=["pyp2rpm", "pyp2spec"],
-        default="pyp2rpm",
+        default="pyp2spec",
     )
 
     parser_pypi_args_optional.add_argument("--template", "-t", dest="spec_template",
@@ -1274,7 +1356,7 @@ def setup_parser():
         "--namespace", dest="namespace", default=None,
         help=(
             "Some DistGit instances (e.g. the Fedora Copr dist-git) use "
-            "a namespaced clone/lookaside URLs.  Typically it meas that "
+            "a namespaced clone/lookaside URLs.  Typically it means that "
             "one package may be hosted in the same DistGit instance "
             "multiple times, in multiple namespaces.  Specify the NAMESPACE "
             "here (e.g. @copr/copr for @copr/copr/copr-cli package)."),
@@ -1337,12 +1419,14 @@ def setup_parser():
               "default is usually set to 5 hours on Copr Frontend."))
     parser_build_parent.add_argument("--nowait", action="store_true", default=False,
                                      help="Don't wait for build")
-    parser_build_parent.add_argument("-r", "--chroot", dest="chroots", action="append",
+    parser_build_parent.add_argument("-r", "--chroot", dest="chroots",
+                                     metavar="CHROOT", action="append",
                                      help="If you don't need this build for all the project's chroots. You can use it several times for each chroot you need.")
 
     parser_build_parent.add_argument(
         "--exclude-chroot",
         dest="exclude_chroots",
+        metavar="CHROOT",
         action="append",
         help=("If you don't need this build for all the project's chroots."
               "You can use it several times for each chroot you don't need.")
@@ -1383,8 +1467,9 @@ def setup_parser():
     # create the parser for the "build" (url/upload) command
     parser_build = subparsers.add_parser("build", parents=[parser_build_parent],
                                          help="Build packages to a specified copr")
-    parser_build.add_argument("pkgs", nargs="+",
-                              help="filename of SRPM or URL of packages to build")
+    parser_build.add_argument(
+        "pkgs", nargs="+",
+        help="List of filenames or URLs for SRPMs or SPEC files to build packages")
 
     parser_build.set_defaults(func="action_build")
 
@@ -1434,7 +1519,8 @@ def setup_parser():
     # create the parser for the "download-build" command
     parser_download_build = subparsers.add_parser("download-build", help="Fetches built packages")
     parser_download_build.add_argument("build_id", help="Build ID")
-    parser_download_build.add_argument("-r", "--chroot", dest="chroots", action="append",
+    parser_download_build.add_argument("-r", "--chroot", dest="chroots",
+                                       metavar="CHROOT", action="append",
                                        help="Select chroots to fetch")
     parser_download_build.add_argument("--dest", "-d", dest="dest",
                                        help="Base directory to store packages", default=".")
@@ -1450,6 +1536,18 @@ def setup_parser():
         action="store_true",
         help="Download only the .spec file",
     )
+    parser_download_build.add_argument(
+        "--review",
+        dest="review",
+        action="store_true",
+        help="Download only the fedora-review files",
+    )
+    parser_download_build.add_argument(
+        "--logs",
+        dest="logs",
+        action="store_true",
+        help="Download only the .log files",
+    )
     parser_download_build.set_defaults(func="action_download_build")
 
     # create the parser for the "cancel" command
@@ -1547,6 +1645,10 @@ def setup_parser():
             "--max-builds",
             help="Keep only the specified number of the newest-by-id builds "\
                  "(garbage collector is run daily), zero disables (default)")
+    parser_add_or_edit_package_parent.add_argument(
+            "--timeout",
+            help="Number of seconds we allow the builds to run, default is "\
+                 "18000 (5h)")
 
     # PyPI edit/create
     parser_add_package_pypi = subparsers.add_parser("add-package-pypi",
